/*
 * Copyright (C) 2003-2007 eXo Platform SAS.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see<http://www.gnu.org/licenses/>.
 */
package bull.com.orderform.jcr;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.UUID;

import javax.jcr.ItemExistsException;
import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.version.VersionException;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.exoplatform.services.log.Log;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.core.ManageableRepository;
import org.exoplatform.services.jcr.impl.core.LocationFactory;
import org.exoplatform.services.log.ExoLogger;
import sun.net.www.MimeTable;

/**
 * Created by The eXo Platform SAS. <br/>
 * 
 * Date: 27.05.2008 <br/>
 * 
 * JavaBean for JCRBrowser sample application.<br/>
 * 
 * Since JCR 1.11 Browser supports Ext Repository synchronization.<br/>
 * 
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: JCRBrowser.java 111 2008-11-11 11:11:11Z peterit $
 */
public class JCRBrowser {
  
  private static final Log                                       LOG                = ExoLogger.getLogger("samples.JCRBrowser");

  protected RepositoryService    repositoryService;

  protected ManageableRepository repository;

  protected Session              session;

  protected Node                 node;

  protected List<String>         errors = new ArrayList<String>();

  /**
   * Get browser repository.
   * 
   * @return the repository
   */
  public ManageableRepository getRepository() {
    return repository;
  }

  /**
   * Set browser repository.
   * 
   * @param repository
   *          the repository to set
   */
  public void setRepository(ManageableRepository repository) {
    this.repository = repository;
  }

  /**
   * Get browser JCR session.
   * 
   * @return the session
   */
  public Session getSession() {
    return session;
  }

  /**
   * Set browser JCR session.
   * 
   * @param session
   *          the session to set
   * @throws RepositoryException
   */
  public void setSession(Session session) throws RepositoryException {
    this.session = session;
    this.node = this.session.getRootNode();
  }

  /**
   * Get browser current node.
   * 
   * @return the node
   */
  public Node getNode() {
    return node;
  }

  /**
   * Set browser current node.
   * 
   * @param node
   *          the node to set
   */
  public void setNode(Node node) {
    this.node = node;
  }

  public void addError(Throwable error) {
    this.errors.add(error.toString());
  }

  public boolean isErrorsFound() {
    return this.errors.size() > 0;
  }

  public String[] getErrorsAndClean() {
    // StringBuilder msg = new StringBuilder();
    // msg.append("<div id='browserErrors' class='errors'>");
    // for (String e: this.errors) {
    // msg.append("&nbsp;&nbsp;&nbsp;&nbsp;");
    // msg.append(e);
    // msg.append("<br/>");
    // }
    // msg.append("</div>");
    // this.errors.clear();
    // return msg.toString();
    try {
      String[] errs = new String[this.errors.size()];
      this.errors.toArray(errs);
      return errs;
    } finally {
      this.errors.clear();
    }
  }

  /**
   * @return the repositoryService
   */
  public RepositoryService getRepositoryService() {
    return repositoryService;
  }

  /**
   * @param repositoryService
   *          the repositoryService to set
   */
  public void setRepositoryService(RepositoryService repositoryService) {
    this.repositoryService = repositoryService;
  }

  /**
   * Tell if Asynchronous Replication service available.
   *
   * @return boolean
   */
  public boolean isAsynchronousReplicationPresent() {
    //return getAsynchronousReplication() != null;
    return false;
  }
  
  /**
   * Tell if Asynchronous Replication service available.
   *
   * @return boolean
   */
  public boolean isAsynchronousReplicationActive() {
    //AsyncReplication arep = getAsynchronousReplication();
    //return arep != null && arep.isActive();
    return false;
  }
  
  /**
   * Runs synchronization process if service is present.
   *
   */
  public void runSynchronization() {
//    AsyncReplicationExecutor arep = getAsynchronousReplicationExecutor();
//    if (arep != null) {
//      try {
//        long start = System.currentTimeMillis();
//        if (arep.synchronize())
//          LOG.info("Synchronization started " + new Date() + ", " + (System.currentTimeMillis() - start) / 1000 + "sec.");
//        else
//          LOG.info("Synchronization star failed. " + (System.currentTimeMillis() - start) / 1000 + "sec.");
//      } catch (AsyncReplicationExecutorException e) {
//        LOG.error("Synchronization fails. Repository error " + e, e);
//      }
//    } else
//      LOG.warn("Synchronization impossible. Service is not present.");
  }
  /**
   * Create orderFormId node with name is orderFormId
   * Every repository must support the primary node type, nt:base. There are a number of other common node types that a repository may support:
   * nt:unstructured is the most flexible node type. It allows any number of child nodes or properties, which can have any names. This node type represents JCRWiki entries.
   * nt:file represents files. It requires a single child node, called jcr:content. This node type represents images and other binary content in a JCRWiki entry.
   * nt:folder node types can represent folders, like those in a conventional filesystem.
   * nt:resource commonly represents the actual content of a file.
   * nt:version is a required node type for repositories that support versioning.
   * @param orderFormId Add orderFormId to node as node name
   * @param description Add  description to the a property of this node
   * @return The node that was added as name <code>orderFormId</code>
   * */
  public Node createOrderFormNode(String orderFormId, String description) {
      try {
          Node chirldNode = this.node.addNode(orderFormId, "nt:folder");
          chirldNode.getSession().save();
          //chirldNode.setProperty("jcr:mixinTypes", "nt:hierarchyNode");
          //chirldNode.setProperty("jcr:description", description);
          //chirldNode.getSession().save();
//          Node node2 = chirldNode.addNode("folderDescription","nt:nodeType");
//          node2.setProperty("jcr:title", "title");
//          node2.setProperty("jcr:description", description);
          return chirldNode;
      } catch (ItemExistsException itemExistsException) {
          // TODO Auto-generated catch block
          itemExistsException.printStackTrace();
      } catch (PathNotFoundException pathNotFoundException) {
          // TODO Auto-generated catch block
          pathNotFoundException.printStackTrace();
      } catch (VersionException vException) {
          // TODO Auto-generated catch block
          vException.printStackTrace();
      } catch (ConstraintViolationException violationException) {
          // TODO Auto-generated catch block
          violationException.printStackTrace();
      } catch (LockException lockException) {
          // TODO Auto-generated catch block
          lockException.printStackTrace();
      } catch (RepositoryException rException) {
          // TODO Auto-generated catch block
          rException.printStackTrace();
      }
      // TODO Auto-generated method stub
      return null;
  }

  /**
   * Save document content as property of node with name is orderFormId
   * @param orderFormId is node name
   * @param binaryData The data add to the property
   * @return <code>Node</code> if binaryData <code>file</code> can add at <code>orderFormId</code>;
   * <code>null</code> otherwise.
   * */
  public Node uploadDocument(String orderFormId, File file, Node nodeFolder) {
      try{
          if(nodeFolder.getName().equals(orderFormId)){
              MimeTable mt = MimeTable.getDefaultTable();
              String mimeType = mt.getContentTypeFor(file.getName());
              if (mimeType == null) mimeType = "application/octet-stream";
              Node fileNode = nodeFolder.addNode(file.getName(), "nt:file");
              //System.out.print("----------fileNode------------> "+fileNode.getPath()+" <------------------------------\n");
              Node resNode = fileNode.addNode("jcr:content", "nt:resource");
              resNode.setProperty("jcr:mimeType", mimeType);
              resNode.setProperty("jcr:encoding", "");
              resNode.setProperty("jcr:data", new FileInputStream(file));
              Calendar lastModified = Calendar.getInstance();
              lastModified.setTimeInMillis(file.lastModified());
              resNode.setProperty("jcr:lastModified", lastModified);
              //System.out.print("----------jcr:mimeType------------> "+resNode.getProperty("jcr:mimeType")+" <------------------------------\n");
              return fileNode;
          }else{
              return null;
          }
      }catch (ItemExistsException itemExistsException) {
          // TODO Auto-generated catch block
          itemExistsException.printStackTrace();
      } catch (PathNotFoundException pathNotFoundException) {
          // TODO Auto-generated catch block
          pathNotFoundException.printStackTrace();
      } catch (VersionException vException) {
          // TODO Auto-generated catch block
          vException.printStackTrace();
      } catch (ConstraintViolationException violationException) {
          // TODO Auto-generated catch block
          violationException.printStackTrace();
      } catch (LockException lockException) {
          // TODO Auto-generated catch block
          lockException.printStackTrace();
      } catch (RepositoryException rException) {
          // TODO Auto-generated catch block
          rException.printStackTrace();
      } catch (FileNotFoundException fileNotFoundException) {
          // TODO Auto-generated catch block
          fileNotFoundException.printStackTrace();
      }
      return null;
  }

  public boolean deleteNode(String path){
      Node nodeDelete;
    try {
        nodeDelete = (Node) this.getSession().getItem(path);
        try {
            Node node2 = nodeDelete.getParent();
            nodeDelete.remove();
            this.setNode(node2);
            return true;
        } catch (VersionException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (LockException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (ConstraintViolationException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (RepositoryException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    } catch (PathNotFoundException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
    } catch (RepositoryException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
    }
    return false;
  }
  /**
   * @param orderFormId
   * @param nodeFolder
   * @param httpRequest
   */
  public void addDocument(String orderFormId, Node nodeFolder, HttpServletRequest httpRequest) {
      //hieuhn BEGIN upload
     boolean isMultipart = ServletFileUpload.isMultipartContent(httpRequest);
     if (!isMultipart) {
         System.out.print("\n----------------------> NO FILE TO PUT <------------------------------\n");
     } else {
        System.out.print("\n----------------------> SOME FILE TO PUT <------------------------------\n");
        FileItemFactory factory = new DiskFileItemFactory();
        ServletFileUpload upload = new ServletFileUpload(factory);
        List items = null;
        try {
            items = upload.parseRequest(httpRequest);
        } catch (FileUploadException e) {
            e.printStackTrace();
        }
        Iterator itr = items.iterator();
        
        System.out.print("\n-------items.size--------> "+items.size()+" <------------------------------\n");
        while (itr.hasNext()) {
            FileItem item = (FileItem) itr.next();
            System.out.print("\n--getFieldName--> "+item.getFieldName()+" <--getName--> "+item.getName()+" <-----\n");
            System.out.print("\n-------item.getSize()--------> "+item.getSize()+" <------------------------------\n");
            if (item.isFormField()) {
                System.out.print("\n---isFormField----> "+item.isFormField()+" <------\n");
            } else {
                try {
                    String itemName = item.getName();
                    String contentType = item.getContentType();
                    long sizeInBytes = item.getSize();
                    InputStream uploadedStream = item.getInputStream();
                    if(nodeFolder.getName().equals(orderFormId) && itemName != null && itemName.length()>0){
                        if (contentType == null) contentType = "--application/octet-stream--";
                        Node fileNode = nodeFolder.addNode(itemName, "nt:file");
                        System.out.print("----------fileNode------------> "+fileNode.getPath()+" <------------------------------\n");
                        System.out.print("\n----------------------> BEGIN FILE <------------------------------\n");
                        System.out.print(uploadedStream);
                        System.out.print("\n----------------------> END FILE <------------------------------\n");
                        Node resNode = fileNode.addNode("jcr:content", "nt:resource");
                        resNode.setProperty("jcr:mimeType", contentType);
                        resNode.setProperty("jcr:encoding", "");
                        Calendar lastModified = Calendar.getInstance();
                        //lastModified.setTimeInMillis(file.lastModified());
                        resNode.setProperty("jcr:lastModified", lastModified);
                        resNode.setProperty("jcr:data", uploadedStream);
                        System.out.print("\n----------------------> END PUT FILE <------------------------------\n");
                        resNode.getSession().save();
                    }
                    uploadedStream.close();
                    System.out.print("\n----------------------> CLOSE FILE <------------------------------\n");
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
       }
     //hieuhn END upload
  }

  public void addNode(String nodeName, HttpServletRequest httpRequest) {
      nodeName = nodeName == null?"new folder":nodeName;
    // TODO Auto-generated method stub
    Node node2 = this.createOrderFormNode(nodeName,"description folder");
    if(node2!=null){
        this.addDocument(nodeName, node2, httpRequest);
        System.out.print("------------------------------> addDocument <------------------------------\n");
    }else{
        System.out.print("------------------------------> Fail to create file node <------------------------------\n");
    }
}
  
  public void addNode(String nodeName, String filePath) {
    // TODO Auto-generated method stub
    Node node2 = this.createOrderFormNode(nodeName,"description folder");
    //File file = new File("e:/java/exo-jcr-1.11.2-tomcat/webapps/browser/user/Sunset.jpg");
    File file = new File(filePath);
    if(node2!=null){
        this.uploadDocument(nodeName, file, node2);
    }else
        System.out.print("------------------------------> Fail to create file node <------------------------------\n");
    /*
    try {
        //Node node2 = this.node.addNode("orderFormId test","description folder");
        //node2.setProperty("jcr:mimeType", "hieuhn test");
        //node2.setProperty("file Type", "file Type hieuhn test");
        //node2.getSession().save();
        //System.out.print("------------------------------> "+node2.getPath()+" <------------------------------\n");
        
        File file = new File("Sunset.jpg");
        MimeTable mt = MimeTable.getDefaultTable();
        String mimeType = mt.getContentTypeFor(file.getName());
        if (mimeType == null) mimeType = "application/octet-stream";
        LocationFactory locationFactory;
        //Value fileContent = new ValueFactoryImpl(locationFactory)

        //Node fileNode = node2.addNode(UUID.randomUUID().toString(), "nt:file");
        Node fileNode = this.node.addNode(UUID.randomUUID().toString(),"nt:file");
        System.out.print("----------fileNode------------> "+fileNode.getPath()+" <------------------------------\n");
        //fileNode.setProperty("jcr:fileName", file.getName());
        fileNode.setProperty("mimeType", mimeType);
        fileNode.setProperty("jcr:encoding", "");
        
        try {
            fileNode.setProperty("jcr:data", new FileInputStream(file));
        } catch (FileNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }finally{
            fileNode.setProperty("jcr:data", "Emplty");
        }
        Calendar lastModified = Calendar.getInstance();
        lastModified.setTimeInMillis(file.lastModified());
        fileNode.setProperty("jcr:lastModified", lastModified);
        fileNode.getSession().save();
        System.out.print("----------fileNode------------> "+fileNode.getProperty("jcr:mimeType")+" <------------------------------\n");

        //this.setNode(node2);
    } catch (ItemExistsException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
    } catch (PathNotFoundException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
    } catch (NoSuchNodeTypeException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
    } catch (LockException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
    } catch (VersionException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
    } catch (ConstraintViolationException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
    } catch (RepositoryException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
    }
    */
}
  
//  /**
//   * Return Asynchronous Replication service handler.
//   *
//   * @return AsyncReplication
//   */
//  private AsyncReplication getAsynchronousReplication() {
//    if (this.repository != null) {
//      WorkspaceContainerFacade container = this.repository.getWorkspaceContainer(this.repository.getConfiguration()
//                                                                                                .getDefaultWorkspaceName());
//      
//      if (container != null)
//        return (AsyncReplication) container.getComponent(AsyncReplication.class);
//    }
//    
//    return null;
//  }
  
//  /**
//   * Return Asynchronous Replication service executor.
//   *
//   * @return AsyncReplication
//   */
//  private AsyncReplicationExecutor getAsynchronousReplicationExecutor() {
//    if (this.repository != null) {
//      WorkspaceContainerFacade container = this.repository.getWorkspaceContainer(this.repository.getConfiguration()
//                                                                                                .getDefaultWorkspaceName());
//      
//      if (container != null)
//        return (AsyncReplicationExecutor) container.getComponent(AsyncReplicationExecutor.class);
//    }
//    
//    return null;
//  }
}
